@@ -101,6 +101,9 @@ class Hybrid_PublicKey : public virtual Botan::Public_Key {
       std::unique_ptr<Botan::Public_Key> m_kem_pk;
 };
 
+BOTAN_DIAGNOSTIC_PUSH
+BOTAN_DIAGNOSTIC_IGNORE_INHERITED_VIA_DOMINANCE
+
 /**
  * This is the private key class for the custom public-key algorithm.
  */
@@ -147,6 +150,8 @@ class Hybrid_PrivateKey : public virtual Botan::Private_Key,
       std::unique_ptr<Botan::Private_Key> m_kem_sk;
 };
 
+BOTAN_DIAGNOSTIC_POP
+
 namespace {
 
 /**
@@ -196,9 +201,9 @@ class Hybrid_Encryption_Operation : public Botan::PK_Ops::KEM_Encryption {
          //      other party, resulting in a shared secret and its encapsulation,
          //   4. Concatenate the ephemeral public key and the encapsulation to
          //      form a "hybrid encapsulation" (to be sent to the other party),
-         //   5. Concatenate the shared secrets of both algorithms and pass the
-         //      result through a user-defined key derivation function to form a
-         //      "hybrid shared secret" (to be used by the application).
+         //   5. Concatenate the shared secrets and ciphertexts of both algorithms
+         //      and pass the result through a user-defined key derivation function
+         //      to form a "hybrid shared secret" (to be used by the application).
 
          // 1. KEX: Generate an ephemeral key pair with the same parameters as
          //         the provided key exchange public key.
@@ -236,9 +241,15 @@ class Hybrid_Encryption_Operation : public Botan::PK_Ops::KEM_Encryption {
          std::copy(
             kem_encapsed_key.begin(), kem_encapsed_key.end(), out_encapsed_key.begin() + kex_encapsed_key.size());
 
-         // 5. Hybrid: Combine the shared secrets of both algorithms.
+         // 5. Hybrid: Combine the shared secrets and ciphertexts of both
+         //            algorithms. Note that there are various known ways for
+         //            such combination logic (see, for example, X-Wing,
+         //            CatKDF, etc.). Applications are encouraged to use a
+         //            well-known KEM-Combiner instead of this example.
          Botan::secure_vector<uint8_t> concat_shared_key;
+         concat_shared_key.insert(concat_shared_key.end(), kex_encapsed_key.begin(), kex_encapsed_key.end());
          concat_shared_key.insert(concat_shared_key.end(), kex_shared_key.begin(), kex_shared_key.end());
+         concat_shared_key.insert(concat_shared_key.end(), kem_encapsed_key.begin(), kem_encapsed_key.end());
          concat_shared_key.insert(concat_shared_key.end(), kem_shared_key.begin(), kem_shared_key.end());
 
          BOTAN_ASSERT_NOMSG(out_shared_key.size() >= desired_shared_key_length);
@@ -301,9 +312,9 @@ class Hybrid_Decryption_Operation : public Botan::PK_Ops::KEM_Decryption {
          //     ephemeral public key (from the other party),
          //  3. Decapsulate a shared secret using the KEM's private key and
          //     the KEM's encapsulation (from the other party),
-         //  4. Concatenate the shared secrets of both algorithms and pass the
-         //     result through a user-defined key derivation function to form a
-         //     "hybrid shared secret" (to be used by the application).
+         //  4. Concatenate the shared secrets and ciphertexts of both algorithms
+         //     and pass the result through a user-defined key derivation function
+         //     to form a "hybrid shared secret" (to be used by the application).
 
          // 1. Hybrid: Extract the ephemeral public key and the encapsulation.
          const auto kex_encapsed_key =
@@ -318,9 +329,15 @@ class Hybrid_Decryption_Operation : public Botan::PK_Ops::KEM_Decryption {
          //         the encapsulation of the other party.
          const auto kem_shared_key = m_kem_decryptor.decrypt(kem_encapsed_key);
 
-         // 4. Hybrid: Combine the shared secrets of both algorithms.
+         // 5. Hybrid: Combine the shared secrets and ciphertexts of both
+         //            algorithms. Note that there are various known ways for
+         //            such combination logic (see, for example, X-Wing,
+         //            CatKDF, etc.). Applications are encouraged to use a
+         //            well-known KEM-Combiner instead of this example.
          Botan::secure_vector<uint8_t> concat_shared_key;
+         concat_shared_key.insert(concat_shared_key.end(), kex_encapsed_key.begin(), kex_encapsed_key.end());
          concat_shared_key.insert(concat_shared_key.end(), kex_shared_key.begin(), kex_shared_key.end());
+         concat_shared_key.insert(concat_shared_key.end(), kem_encapsed_key.begin(), kem_encapsed_key.end());
          concat_shared_key.insert(concat_shared_key.end(), kem_shared_key.begin(), kem_shared_key.end());
 
          BOTAN_ASSERT_NOMSG(out_shared_key.size() >= desired_shared_key_length);
@@ -355,12 +372,12 @@ int main() {
 
    // Alice generates two key pairs suitable for:
    //   1) key exchange (X25519), and
-   //   2) key encapsulation (Kyber).
+   //   2) key encapsulation (ML-KEM).
    //
    // She then combines them into a custom "hybrid" key pair that acts
    // like a key encapsulation mechanism (KEM).
    const auto private_key_of_alice = std::make_unique<Hybrid_PrivateKey>(
-      Botan::create_private_key("X25519", rng), Botan::create_private_key("Kyber", rng, "Kyber-768-r3"));
+      Botan::create_private_key("X25519", rng), Botan::create_private_key("ML-KEM", rng, "ML-KEM-768"));
    const auto public_key_of_alice = private_key_of_alice->public_key();
 
    // Bob uses Alice's public key to encapsulate a shared secret, and
