@@ -17,14 +17,8 @@
 
 namespace Botan {
 
-#if defined(BOTAN_USE_GCC_INLINE_ASM)
-   #if defined(BOTAN_TARGET_ARCH_IS_X86_FAMILY)
-      #define BOTAN_MP_USE_X86_32_ASM
-   #endif
-
-   #if defined(BOTAN_TARGET_ARCH_IS_X86_64)
-      #define BOTAN_MP_USE_X86_64_ASM
-   #endif
+#if defined(BOTAN_USE_GCC_INLINE_ASM) && defined(BOTAN_TARGET_ARCH_IS_X86_64)
+   #define BOTAN_MP_USE_X86_64_ASM
 #endif
 
 /*
@@ -42,6 +36,7 @@ struct WordInfo<uint32_t> {
       static const constexpr size_t bytes = 4;
       static const constexpr size_t bits = 32;
       static const constexpr uint32_t max = 0xFFFFFFFF;
+      static const constexpr uint32_t top_bit = 0x80000000;
 
       typedef uint64_t dword;
       static const constexpr bool dword_is_native = true;
@@ -53,6 +48,7 @@ struct WordInfo<uint64_t> {
       static const constexpr size_t bytes = 8;
       static const constexpr size_t bits = 64;
       static const constexpr uint64_t max = 0xFFFFFFFFFFFFFFFF;
+      static const constexpr uint64_t top_bit = 0x8000000000000000;
 
 #if defined(BOTAN_TARGET_HAS_NATIVE_UINT128)
       typedef uint128_t dword;
@@ -68,21 +64,6 @@ struct WordInfo<uint64_t> {
 */
 template <WordType W>
 inline constexpr auto word_madd2(W a, W b, W* c) -> W {
-#if defined(BOTAN_MP_USE_X86_32_ASM)
-   if(std::same_as<W, uint32_t> && !std::is_constant_evaluated()) {
-      asm(R"(
-         mull %[b]
-         addl %[c],%[a]
-         adcl $0,%[carry]
-         )"
-          : [a] "=a"(a), [b] "=rm"(b), [carry] "=&d"(*c)
-          : "0"(a), "1"(b), [c] "g"(*c)
-          : "cc");
-
-      return a;
-   }
-#endif
-
 #if defined(BOTAN_MP_USE_X86_64_ASM)
    if(std::same_as<W, uint64_t> && !std::is_constant_evaluated()) {
       asm(R"(
@@ -109,25 +90,6 @@ inline constexpr auto word_madd2(W a, W b, W* c) -> W {
 */
 template <WordType W>
 inline constexpr auto word_madd3(W a, W b, W c, W* d) -> W {
-#if defined(BOTAN_MP_USE_X86_32_ASM)
-   if(std::same_as<W, uint32_t> && !std::is_constant_evaluated()) {
-      asm(R"(
-         mull %[b]
-
-         addl %[c],%[a]
-         adcl $0,%[carry]
-
-         addl %[d],%[a]
-         adcl $0,%[carry]
-         )"
-          : [a] "=a"(a), [b] "=rm"(b), [carry] "=&d"(*d)
-          : "0"(a), "1"(b), [c] "g"(c), [d] "g"(*d)
-          : "cc");
-
-      return a;
-   }
-#endif
-
 #if defined(BOTAN_MP_USE_X86_64_ASM)
    if(std::same_as<W, uint64_t> && !std::is_constant_evaluated()) {
       asm(R"(
@@ -153,42 +115,25 @@ inline constexpr auto word_madd3(W a, W b, W c, W* d) -> W {
    return static_cast<W>(s);
 }
 
-#if defined(BOTAN_MP_USE_X86_32_ASM)
+#if defined(BOTAN_MP_USE_X86_64_ASM)
 
-   #define ADDSUB2_OP(OPERATION, INDEX)        \
-      ASM("movl 4*" #INDEX "(%[y]), %[carry]") \
-      ASM(OPERATION " %[carry], 4*" #INDEX "(%[x])")
+   #define ASM(x) x "\n\t"
 
-   #define ADDSUB3_OP(OPERATION, INDEX)              \
-      ASM("movl 4*" #INDEX "(%[x]), %[carry]")       \
-      ASM(OPERATION " 4*" #INDEX "(%[y]), %[carry]") \
-      ASM("movl %[carry], 4*" #INDEX "(%[z])")
+   #define DO_4_TIMES(MACRO, ARG) \
+      MACRO(ARG, 0)               \
+      MACRO(ARG, 1)               \
+      MACRO(ARG, 2)               \
+      MACRO(ARG, 3)
 
-   #define LINMUL_OP(WRITE_TO, INDEX)      \
-      ASM("movl 4*" #INDEX "(%[x]),%%eax") \
-      ASM("mull %[y]")                     \
-      ASM("addl %[carry],%%eax")           \
-      ASM("adcl $0,%%edx")                 \
-      ASM("movl %%edx,%[carry]")           \
-      ASM("movl %%eax, 4*" #INDEX "(%[" WRITE_TO "])")
-
-   #define MULADD_OP(IGNORED, INDEX)       \
-      ASM("movl 4*" #INDEX "(%[x]),%%eax") \
-      ASM("mull %[y]")                     \
-      ASM("addl %[carry],%%eax")           \
-      ASM("adcl $0,%%edx")                 \
-      ASM("addl 4*" #INDEX "(%[z]),%%eax") \
-      ASM("adcl $0,%%edx")                 \
-      ASM("movl %%edx,%[carry]")           \
-      ASM("movl %%eax, 4*" #INDEX " (%[z])")
-
-   #define ADD_OR_SUBTRACT(CORE_CODE) \
-      ASM("rorl %[carry]")            \
-      CORE_CODE                       \
-      ASM("sbbl %[carry],%[carry]")   \
-      ASM("negl %[carry]")
-
-#elif defined(BOTAN_MP_USE_X86_64_ASM)
+   #define DO_8_TIMES(MACRO, ARG) \
+      MACRO(ARG, 0)               \
+      MACRO(ARG, 1)               \
+      MACRO(ARG, 2)               \
+      MACRO(ARG, 3)               \
+      MACRO(ARG, 4)               \
+      MACRO(ARG, 5)               \
+      MACRO(ARG, 6)               \
+      MACRO(ARG, 7)
 
    #define ADDSUB2_OP(OPERATION, INDEX)        \
       ASM("movq 8*" #INDEX "(%[y]), %[carry]") \
@@ -225,51 +170,36 @@ inline constexpr auto word_madd3(W a, W b, W c, W* d) -> W {
 
 #endif
 
-#if defined(ADD_OR_SUBTRACT)
-
-   #define ASM(x) x "\n\t"
-
-   #define DO_8_TIMES(MACRO, ARG) \
-      MACRO(ARG, 0)               \
-      MACRO(ARG, 1)               \
-      MACRO(ARG, 2)               \
-      MACRO(ARG, 3)               \
-      MACRO(ARG, 4)               \
-      MACRO(ARG, 5)               \
-      MACRO(ARG, 6)               \
-      MACRO(ARG, 7)
-
-#endif
-
 /*
 * Word Addition
 */
 template <WordType W>
 inline constexpr auto word_add(W x, W y, W* carry) -> W {
-#if defined(BOTAN_MP_USE_X86_32_ASM)
-   if(std::same_as<W, uint32_t> && !std::is_constant_evaluated()) {
-      asm(ADD_OR_SUBTRACT(ASM("adcl %[y],%[x]"))
-          : [x] "=r"(x), [carry] "=r"(*carry)
-          : "0"(x), [y] "rm"(y), "1"(*carry)
-          : "cc");
-      return x;
-   }
+   if(!std::is_constant_evaluated()) {
+#if BOTAN_COMPILER_HAS_BUILTIN(__builtin_addc)
+      if constexpr(std::same_as<W, unsigned int>) {
+         return __builtin_addc(x, y, *carry & 1, carry);
+      } else if constexpr(std::same_as<W, unsigned long>) {
+         return __builtin_addcl(x, y, *carry & 1, carry);
+      } else if constexpr(std::same_as<W, unsigned long long>) {
+         return __builtin_addcll(x, y, *carry & 1, carry);
+      }
+#elif defined(BOTAN_MP_USE_X86_64_ASM)
+      if(std::same_as<W, uint64_t>) {
+         asm(ADD_OR_SUBTRACT(ASM("adcq %[y],%[x]"))
+             : [x] "=r"(x), [carry] "=r"(*carry)
+             : "0"(x), [y] "rm"(y), "1"(*carry)
+             : "cc");
+         return x;
+      }
 #endif
-
-#if defined(BOTAN_MP_USE_X86_64_ASM)
-   if(std::same_as<W, uint64_t> && !std::is_constant_evaluated()) {
-      asm(ADD_OR_SUBTRACT(ASM("adcq %[y],%[x]"))
-          : [x] "=r"(x), [carry] "=r"(*carry)
-          : "0"(x), [y] "rm"(y), "1"(*carry)
-          : "cc");
-      return x;
    }
-#endif
 
+   const W cb = *carry & 1;
    W z = x + y;
    W c1 = (z < x);
-   z += *carry;
-   *carry = c1 | (z < *carry);
+   z += cb;
+   *carry = c1 | (z < cb);
    return z;
 }
 
@@ -278,16 +208,6 @@ inline constexpr auto word_add(W x, W y, W* carry) -> W {
 */
 template <WordType W>
 inline constexpr auto word8_add2(W x[8], const W y[8], W carry) -> W {
-#if defined(BOTAN_MP_USE_X86_32_ASM)
-   if(std::same_as<W, uint32_t> && !std::is_constant_evaluated()) {
-      asm(ADD_OR_SUBTRACT(DO_8_TIMES(ADDSUB2_OP, "adcl"))
-          : [carry] "=r"(carry)
-          : [x] "r"(x), [y] "r"(y), "0"(carry)
-          : "cc", "memory");
-      return carry;
-   }
-#endif
-
 #if defined(BOTAN_MP_USE_X86_64_ASM)
    if(std::same_as<W, uint64_t> && !std::is_constant_evaluated()) {
       asm(ADD_OR_SUBTRACT(DO_8_TIMES(ADDSUB2_OP, "adcq"))
@@ -314,22 +234,12 @@ inline constexpr auto word8_add2(W x[8], const W y[8], W carry) -> W {
 */
 template <WordType W>
 inline constexpr auto word8_add3(W z[8], const W x[8], const W y[8], W carry) -> W {
-#if defined(BOTAN_MP_USE_X86_32_ASM)
-   if(std::same_as<W, uint32_t> && !std::is_constant_evaluated()) {
-      asm(ADD_OR_SUBTRACT(DO_8_TIMES(ADDSUB3_OP, "adcl"))
-          : [carry] "=r"(carry)
-          : [x] "r"(x), [y] "r"(y), [z] "r"(z), "0"(carry)
-          : "cc", "memory");
-      return carry;
-   }
-#endif
-
 #if defined(BOTAN_MP_USE_X86_64_ASM)
    if(std::same_as<W, uint64_t> && !std::is_constant_evaluated()) {
-      asm(ADD_OR_SUBTRACT(DO_8_TIMES(ADDSUB3_OP, "adcq"))
-          : [carry] "=r"(carry)
-          : [x] "r"(x), [y] "r"(y), [z] "r"(z), "0"(carry)
-          : "cc", "memory");
+      asm volatile(ADD_OR_SUBTRACT(DO_8_TIMES(ADDSUB3_OP, "adcq"))
+                   : [carry] "=r"(carry)
+                   : [x] "r"(x), [y] "r"(y), [z] "r"(z), "0"(carry)
+                   : "cc", "memory");
       return carry;
    }
 #endif
@@ -345,34 +255,54 @@ inline constexpr auto word8_add3(W z[8], const W x[8], const W y[8], W carry) ->
    return carry;
 }
 
+template <WordType W>
+inline constexpr auto word4_add3(W z[4], const W x[4], const W y[4], W carry) -> W {
+#if defined(BOTAN_MP_USE_X86_64_ASM)
+   if(std::same_as<W, uint64_t> && !std::is_constant_evaluated()) {
+      asm volatile(ADD_OR_SUBTRACT(DO_4_TIMES(ADDSUB3_OP, "adcq"))
+                   : [carry] "=r"(carry)
+                   : [x] "r"(x), [y] "r"(y), [z] "r"(z), "0"(carry)
+                   : "cc", "memory");
+      return carry;
+   }
+#endif
+
+   z[0] = word_add(x[0], y[0], &carry);
+   z[1] = word_add(x[1], y[1], &carry);
+   z[2] = word_add(x[2], y[2], &carry);
+   z[3] = word_add(x[3], y[3], &carry);
+   return carry;
+}
+
 /*
 * Word Subtraction
 */
 template <WordType W>
 inline constexpr auto word_sub(W x, W y, W* carry) -> W {
-#if defined(BOTAN_MP_USE_X86_32_ASM)
-   if(std::same_as<W, uint32_t> && !std::is_constant_evaluated()) {
-      asm(ADD_OR_SUBTRACT(ASM("sbbl %[y],%[x]"))
-          : [x] "=r"(x), [carry] "=r"(*carry)
-          : "0"(x), [y] "rm"(y), "1"(*carry)
-          : "cc");
-      return x;
-   }
+   if(!std::is_constant_evaluated()) {
+#if BOTAN_COMPILER_HAS_BUILTIN(__builtin_subc)
+      if constexpr(std::same_as<W, unsigned int>) {
+         return __builtin_subc(x, y, *carry & 1, carry);
+      } else if constexpr(std::same_as<W, unsigned long>) {
+         return __builtin_subcl(x, y, *carry & 1, carry);
+      } else if constexpr(std::same_as<W, unsigned long long>) {
+         return __builtin_subcll(x, y, *carry & 1, carry);
+      }
+#elif defined(BOTAN_MP_USE_X86_64_ASM)
+      if(std::same_as<W, uint64_t>) {
+         asm(ADD_OR_SUBTRACT(ASM("sbbq %[y],%[x]"))
+             : [x] "=r"(x), [carry] "=r"(*carry)
+             : "0"(x), [y] "rm"(y), "1"(*carry)
+             : "cc");
+         return x;
+      }
 #endif
-
-#if defined(BOTAN_MP_USE_X86_64_ASM)
-   if(std::same_as<W, uint64_t> && !std::is_constant_evaluated()) {
-      asm(ADD_OR_SUBTRACT(ASM("sbbq %[y],%[x]"))
-          : [x] "=r"(x), [carry] "=r"(*carry)
-          : "0"(x), [y] "rm"(y), "1"(*carry)
-          : "cc");
-      return x;
    }
-#endif
 
+   const W cb = *carry & 1;
    W t0 = x - y;
    W c1 = (t0 > x);
-   W z = t0 - *carry;
+   W z = t0 - cb;
    *carry = c1 | (z > t0);
    return z;
 }
@@ -382,16 +312,6 @@ inline constexpr auto word_sub(W x, W y, W* carry) -> W {
 */
 template <WordType W>
 inline constexpr auto word8_sub2(W x[8], const W y[8], W carry) -> W {
-#if defined(BOTAN_MP_USE_X86_32_ASM)
-   if(std::same_as<W, uint32_t> && !std::is_constant_evaluated()) {
-      asm(ADD_OR_SUBTRACT(DO_8_TIMES(ADDSUB2_OP, "sbbl"))
-          : [carry] "=r"(carry)
-          : [x] "r"(x), [y] "r"(y), "0"(carry)
-          : "cc", "memory");
-      return carry;
-   }
-#endif
-
 #if defined(BOTAN_MP_USE_X86_64_ASM)
    if(std::same_as<W, uint64_t> && !std::is_constant_evaluated()) {
       asm(ADD_OR_SUBTRACT(DO_8_TIMES(ADDSUB2_OP, "sbbq"))
@@ -418,16 +338,6 @@ inline constexpr auto word8_sub2(W x[8], const W y[8], W carry) -> W {
 */
 template <WordType W>
 inline constexpr auto word8_sub2_rev(W x[8], const W y[8], W carry) -> W {
-#if defined(BOTAN_MP_USE_X86_32_ASM)
-   if(std::same_as<W, uint32_t> && !std::is_constant_evaluated()) {
-      asm(ADD_OR_SUBTRACT(DO_8_TIMES(ADDSUB3_OP, "sbbl"))
-          : [carry] "=r"(carry)
-          : [x] "r"(y), [y] "r"(x), [z] "r"(x), "0"(carry)
-          : "cc", "memory");
-      return carry;
-   }
-#endif
-
 #if defined(BOTAN_MP_USE_X86_64_ASM)
    if(std::same_as<W, uint64_t> && !std::is_constant_evaluated()) {
       asm(ADD_OR_SUBTRACT(DO_8_TIMES(ADDSUB3_OP, "sbbq"))
@@ -454,22 +364,12 @@ inline constexpr auto word8_sub2_rev(W x[8], const W y[8], W carry) -> W {
 */
 template <WordType W>
 inline constexpr auto word8_sub3(W z[8], const W x[8], const W y[8], W carry) -> W {
-#if defined(BOTAN_MP_USE_X86_32_ASM)
-   if(std::same_as<W, uint32_t> && !std::is_constant_evaluated()) {
-      asm(ADD_OR_SUBTRACT(DO_8_TIMES(ADDSUB3_OP, "sbbl"))
-          : [carry] "=r"(carry)
-          : [x] "r"(x), [y] "r"(y), [z] "r"(z), "0"(carry)
-          : "cc", "memory");
-      return carry;
-   }
-#endif
-
 #if defined(BOTAN_MP_USE_X86_64_ASM)
    if(std::same_as<W, uint64_t> && !std::is_constant_evaluated()) {
-      asm(ADD_OR_SUBTRACT(DO_8_TIMES(ADDSUB3_OP, "sbbq"))
-          : [carry] "=r"(carry)
-          : [x] "r"(x), [y] "r"(y), [z] "r"(z), "0"(carry)
-          : "cc", "memory");
+      asm volatile(ADD_OR_SUBTRACT(DO_8_TIMES(ADDSUB3_OP, "sbbq"))
+                   : [carry] "=r"(carry)
+                   : [x] "r"(x), [y] "r"(y), [z] "r"(z), "0"(carry)
+                   : "cc", "memory");
       return carry;
    }
 #endif
@@ -485,21 +385,30 @@ inline constexpr auto word8_sub3(W z[8], const W x[8], const W y[8], W carry) ->
    return carry;
 }
 
+template <WordType W>
+inline constexpr auto word4_sub3(W z[4], const W x[4], const W y[4], W carry) -> W {
+#if defined(BOTAN_MP_USE_X86_64_ASM)
+   if(std::same_as<W, uint64_t> && !std::is_constant_evaluated()) {
+      asm volatile(ADD_OR_SUBTRACT(DO_4_TIMES(ADDSUB3_OP, "sbbq"))
+                   : [carry] "=r"(carry)
+                   : [x] "r"(x), [y] "r"(y), [z] "r"(z), "0"(carry)
+                   : "cc", "memory");
+      return carry;
+   }
+#endif
+
+   z[0] = word_sub(x[0], y[0], &carry);
+   z[1] = word_sub(x[1], y[1], &carry);
+   z[2] = word_sub(x[2], y[2], &carry);
+   z[3] = word_sub(x[3], y[3], &carry);
+   return carry;
+}
+
 /*
 * Eight Word Block Linear Multiplication
 */
 template <WordType W>
 inline constexpr auto word8_linmul2(W x[8], W y, W carry) -> W {
-#if defined(BOTAN_MP_USE_X86_32_ASM)
-   if(std::same_as<W, uint32_t> && !std::is_constant_evaluated()) {
-      asm(DO_8_TIMES(LINMUL_OP, "x")
-          : [carry] "=r"(carry)
-          : [x] "r"(x), [y] "rm"(y), "0"(carry)
-          : "cc", "%eax", "%edx");
-      return carry;
-   }
-#endif
-
 #if defined(BOTAN_MP_USE_X86_64_ASM)
    if(std::same_as<W, uint64_t> && !std::is_constant_evaluated()) {
       asm(DO_8_TIMES(LINMUL_OP, "x")
@@ -526,16 +435,6 @@ inline constexpr auto word8_linmul2(W x[8], W y, W carry) -> W {
 */
 template <WordType W>
 inline constexpr auto word8_linmul3(W z[8], const W x[8], W y, W carry) -> W {
-#if defined(BOTAN_MP_USE_X86_32_ASM)
-   if(std::same_as<W, uint32_t> && !std::is_constant_evaluated()) {
-      asm(DO_8_TIMES(LINMUL_OP, "z")
-          : [carry] "=r"(carry)
-          : [z] "r"(z), [x] "r"(x), [y] "rm"(y), "0"(carry)
-          : "cc", "%eax", "%edx");
-      return carry;
-   }
-#endif
-
 #if defined(BOTAN_MP_USE_X86_64_ASM)
    if(std::same_as<W, uint64_t> && !std::is_constant_evaluated()) {
       asm(DO_8_TIMES(LINMUL_OP, "z")
@@ -562,16 +461,6 @@ inline constexpr auto word8_linmul3(W z[8], const W x[8], W y, W carry) -> W {
 */
 template <WordType W>
 inline constexpr auto word8_madd3(W z[8], const W x[8], W y, W carry) -> W {
-#if defined(BOTAN_MP_USE_X86_32_ASM)
-   if(std::same_as<W, uint32_t> && !std::is_constant_evaluated()) {
-      asm(DO_8_TIMES(MULADD_OP, "")
-          : [carry] "=r"(carry)
-          : [z] "r"(z), [x] "r"(x), [y] "rm"(y), "0"(carry)
-          : "cc", "%eax", "%edx");
-      return carry;
-   }
-#endif
-
 #if defined(BOTAN_MP_USE_X86_64_ASM)
    if(std::same_as<W, uint64_t> && !std::is_constant_evaluated()) {
       asm(DO_8_TIMES(MULADD_OP, "")
@@ -599,24 +488,6 @@ inline constexpr auto word8_madd3(W z[8], const W x[8], W y, W carry) -> W {
 */
 template <WordType W>
 inline constexpr void word3_muladd(W* w2, W* w1, W* w0, W x, W y) {
-#if defined(BOTAN_MP_USE_X86_32_ASM)
-   if(std::same_as<W, uint32_t> && !std::is_constant_evaluated()) {
-      W z0 = 0, z1 = 0;
-
-      asm("mull %[y]" : "=a"(z0), "=d"(z1) : "a"(x), [y] "rm"(y) : "cc");
-
-      asm(R"(
-          addl %[z0],%[w0]
-          adcl %[z1],%[w1]
-          adcl $0,%[w2]
-          )"
-          : [w0] "=r"(*w0), [w1] "=r"(*w1), [w2] "=r"(*w2)
-          : [z0] "r"(z0), [z1] "r"(z1), "0"(*w0), "1"(*w1), "2"(*w2)
-          : "cc");
-      return;
-   }
-#endif
-
 #if defined(BOTAN_MP_USE_X86_64_ASM)
    if(std::same_as<W, uint64_t> && !std::is_constant_evaluated()) {
       W z0 = 0, z1 = 0;
@@ -647,20 +518,6 @@ inline constexpr void word3_muladd(W* w2, W* w1, W* w0, W x, W y) {
 */
 template <WordType W>
 inline constexpr void word3_add(W* w2, W* w1, W* w0, W x) {
-#if defined(BOTAN_MP_USE_X86_32_ASM)
-   if(std::same_as<W, uint32_t> && !std::is_constant_evaluated()) {
-      asm(R"(
-         addl %[x],%[w0]
-         adcl $0,%[w1]
-         adcl $0,%[w2]
-         )"
-          : [w0] "=r"(*w0), [w1] "=r"(*w1), [w2] "=r"(*w2)
-          : [x] "r"(x), "0"(*w0), "1"(*w1), "2"(*w2)
-          : "cc");
-      return;
-   }
-#endif
-
 #if defined(BOTAN_MP_USE_X86_64_ASM)
    if(std::same_as<W, uint64_t> && !std::is_constant_evaluated()) {
       asm(R"(
@@ -688,28 +545,6 @@ inline constexpr void word3_add(W* w2, W* w1, W* w0, W x) {
 */
 template <WordType W>
 inline constexpr void word3_muladd_2(W* w2, W* w1, W* w0, W x, W y) {
-#if defined(BOTAN_MP_USE_X86_32_ASM)
-   if(std::same_as<W, uint32_t> && !std::is_constant_evaluated()) {
-      W z0 = 0, z1 = 0;
-
-      asm("mull %[y]" : "=a"(z0), "=d"(z1) : "a"(x), [y] "rm"(y) : "cc");
-
-      asm(R"(
-         addl %[z0],%[w0]
-         adcl %[z1],%[w1]
-         adcl $0,%[w2]
-
-         addl %[z0],%[w0]
-         adcl %[z1],%[w1]
-         adcl $0,%[w2]
-         )"
-          : [w0] "=r"(*w0), [w1] "=r"(*w1), [w2] "=r"(*w2)
-          : [z0] "r"(z0), [z1] "r"(z1), "0"(*w0), "1"(*w1), "2"(*w2)
-          : "cc");
-      return;
-   }
-#endif
-
 #if defined(BOTAN_MP_USE_X86_64_ASM)
    if(std::same_as<W, uint64_t> && !std::is_constant_evaluated()) {
       W z0 = 0, z1 = 0;
@@ -749,8 +584,100 @@ inline constexpr void word3_muladd_2(W* w2, W* w1, W* w0, W x, W y) {
    *w2 = word_add(*w2, top, &carry);
 }
 
+/**
+* Helper for 3-word accumulators
+*
+* A number of algorithms especially Comba multiplication and
+* Montgomery reduction can take advantage of wide accumulators, which
+* consume inputs via addition with outputs extracted from the low
+* bits.
+*/
+template <WordType W>
+class word3 {
+#if defined(__BITINT_MAXWIDTH__) && (__BITINT_MAXWIDTH__ >= 3 * 64)
+
+   public:
+      constexpr word3() { m_w = 0; }
+
+      inline constexpr void mul(W x, W y) { m_w += static_cast<W3>(x) * y; }
+
+      inline constexpr void mul_x2(W x, W y) { m_w += static_cast<W3>(x) * y * 2; }
+
+      inline constexpr void add(W x) { m_w += x; }
+
+      inline constexpr W extract() {
+         W r = static_cast<W>(m_w);
+         m_w >>= WordInfo<W>::bits;
+         return r;
+      }
+
+      inline constexpr W monty_step(W p0, W p_dash) {
+         const W w0 = static_cast<W>(m_w);
+         const W r = w0 * p_dash;
+         mul(r, p0);
+         m_w >>= WordInfo<W>::bits;
+         return r;
+      }
+
+      inline constexpr W monty_step_pdash1() {
+         const W r = static_cast<W>(m_w);
+         m_w >>= WordInfo<W>::bits;
+         m_w += static_cast<W3>(r);
+         return r;
+      }
+
+   private:
+      __extension__ typedef unsigned _BitInt(WordInfo<W>::bits * 3) W3;
+      W3 m_w;
+#else
+
+   public:
+      constexpr word3() {
+         m_w2 = 0;
+         m_w1 = 0;
+         m_w0 = 0;
+      }
+
+      inline constexpr void mul(W x, W y) { word3_muladd(&m_w2, &m_w1, &m_w0, x, y); }
+
+      inline constexpr void mul_x2(W x, W y) { word3_muladd_2(&m_w2, &m_w1, &m_w0, x, y); }
+
+      inline constexpr void add(W x) { word3_add(&m_w2, &m_w1, &m_w0, x); }
+
+      inline constexpr W extract() {
+         W r = m_w0;
+         m_w0 = m_w1;
+         m_w1 = m_w2;
+         m_w2 = 0;
+         return r;
+      }
+
+      inline constexpr W monty_step(W p0, W p_dash) {
+         W r = m_w0 * p_dash;
+         mul(r, p0);
+         m_w0 = m_w1;
+         m_w1 = m_w2;
+         m_w2 = 0;
+         return r;
+      }
+
+      inline constexpr W monty_step_pdash1() {
+         // If p_dash == 1 then p[0] = -1 and everything simplifies
+         const W r = m_w0;
+         m_w0 += m_w1;
+         m_w1 = m_w2 + (m_w0 < m_w1);
+         m_w2 = 0;
+         return r;
+      }
+
+   private:
+      W m_w0, m_w1, m_w2;
+#endif
+};
+
 #if defined(ASM)
    #undef ASM
+   #undef DO_4_TIMES
    #undef DO_8_TIMES
    #undef ADD_OR_SUBTRACT
    #undef ADDSUB2_OP
