@@ -82,6 +82,10 @@
    #include <botan/kyber.h>
 #endif
 
+#if defined(BOTAN_HAS_ML_KEM)
+   #include <botan/ml_kem.h>
+#endif
+
 #if defined(BOTAN_HAS_HSS_LMS)
    #include <botan/hss_lms.h>
 #endif
@@ -98,10 +102,18 @@
    #include <botan/dilithium.h>
 #endif
 
+#if defined(BOTAN_HAS_ML_DSA)
+   #include <botan/ml_dsa.h>
+#endif
+
 #if defined(BOTAN_HAS_SPHINCS_PLUS_WITH_SHA2) || defined(BOTAN_HAS_SPHINCS_PLUS_WITH_SHAKE)
    #include <botan/sphincsplus.h>
 #endif
 
+#if defined(BOTAN_HAS_SLH_DSA_WITH_SHA2) || defined(BOTAN_HAS_SLH_DSA_WITH_SHAKE)
+   #include <botan/slh_dsa.h>
+#endif
+
 namespace Botan {
 
 std::unique_ptr<Public_Key> load_public_key(const AlgorithmIdentifier& alg_id,
@@ -146,6 +158,12 @@ std::unique_ptr<Public_Key> load_public_key(const AlgorithmIdentifier& alg_id,
    }
 #endif
 
+#if defined(BOTAN_HAS_ML_KEM)
+   if(alg_name.starts_with("ML-KEM-")) {
+      return std::make_unique<ML_KEM_PublicKey>(alg_id, key_bits);
+   }
+#endif
+
 #if defined(BOTAN_HAS_ECDSA)
    if(alg_name == "ECDSA") {
       return std::make_unique<ECDSA_PublicKey>(alg_id, key_bits);
@@ -224,6 +242,12 @@ std::unique_ptr<Public_Key> load_public_key(const AlgorithmIdentifier& alg_id,
    }
 #endif
 
+#if defined(BOTAN_HAS_ML_DSA)
+   if(alg_name.starts_with("ML-DSA-")) {
+      return std::make_unique<ML_DSA_PublicKey>(alg_id, key_bits);
+   }
+#endif
+
 #if defined(BOTAN_HAS_HSS_LMS)
    if(alg_name == "HSS-LMS") {
       return std::make_unique<HSS_LMS_PublicKey>(key_bits);
@@ -236,6 +260,12 @@ std::unique_ptr<Public_Key> load_public_key(const AlgorithmIdentifier& alg_id,
    }
 #endif
 
+#if defined(BOTAN_HAS_SLH_DSA_WITH_SHA2) || defined(BOTAN_HAS_SLH_DSA_WITH_SHAKE)
+   if(alg_name.starts_with("SLH-DSA-") || alg_name.starts_with("Hash-SLH-DSA-")) {
+      return std::make_unique<SLH_DSA_PublicKey>(alg_id, key_bits);
+   }
+#endif
+
    throw Decoding_Error(fmt("Unknown or unavailable public key algorithm '{}'", alg_name));
 }
 
@@ -299,6 +329,12 @@ std::unique_ptr<Private_Key> load_private_key(const AlgorithmIdentifier& alg_id,
    }
 #endif
 
+#if defined(BOTAN_HAS_ML_KEM)
+   if(alg_name.starts_with("ML-KEM-")) {
+      return std::make_unique<ML_KEM_PrivateKey>(alg_id, key_bits);
+   }
+#endif
+
 #if defined(BOTAN_HAS_MCELIECE)
    if(alg_name == "McEliece") {
       return std::make_unique<McEliece_PrivateKey>(key_bits);
@@ -359,6 +395,12 @@ std::unique_ptr<Private_Key> load_private_key(const AlgorithmIdentifier& alg_id,
    }
 #endif
 
+#if defined(BOTAN_HAS_ML_DSA)
+   if(alg_name.starts_with("ML-DSA-")) {
+      return std::make_unique<ML_DSA_PrivateKey>(alg_id, key_bits);
+   }
+#endif
+
 #if defined(BOTAN_HAS_HSS_LMS)
    if(alg_name == "HSS-LMS-Private-Key") {
       return std::make_unique<HSS_LMS_PrivateKey>(key_bits);
@@ -371,6 +413,12 @@ std::unique_ptr<Private_Key> load_private_key(const AlgorithmIdentifier& alg_id,
    }
 #endif
 
+#if defined(BOTAN_HAS_SLH_DSA_WITH_SHA2) || defined(BOTAN_HAS_SLH_DSA_WITH_SHAKE)
+   if(alg_name.starts_with("SLH-DSA-") || alg_name.starts_with("Hash-SLH-DSA-")) {
+      return std::make_unique<SLH_DSA_PrivateKey>(alg_id, key_bits);
+   }
+#endif
+
    throw Decoding_Error(fmt("Unknown or unavailable public key algorithm '{}'", alg_name));
 }
 
@@ -488,8 +536,21 @@ std::unique_ptr<Private_Key> create_private_key(std::string_view alg_name,
    }
 #endif
 
+#if defined(BOTAN_HAS_ML_KEM)
+   if(alg_name == "ML-KEM") {
+      const auto mode = [&]() -> ML_KEM_Mode {
+         if(params.empty()) {
+            return ML_KEM_Mode::ML_KEM_768;
+         }
+         return ML_KEM_Mode(params);
+      }();
+
+      return std::make_unique<ML_KEM_PrivateKey>(rng, mode);
+   }
+#endif
+
 #if defined(BOTAN_HAS_DILITHIUM) || defined(BOTAN_HAS_DILITHIUM_AES)
-   if(alg_name == "Dilithium" || alg_name == "Dilithium-") {
+   if(alg_name == "Dilithium" || alg_name.starts_with("Dilithium-")) {
       const auto mode = [&]() -> DilithiumMode {
          if(params.empty()) {
             return DilithiumMode::Dilithium6x5;
@@ -501,6 +562,19 @@ std::unique_ptr<Private_Key> create_private_key(std::string_view alg_name,
    }
 #endif
 
+#if defined(BOTAN_HAS_ML_DSA)
+   if(alg_name == "ML-DSA") {
+      const auto mode = [&]() -> ML_DSA_Mode {
+         if(params.empty()) {
+            return ML_DSA_Mode::ML_DSA_6x5;
+         }
+         return ML_DSA_Mode(params);
+      }();
+
+      return std::make_unique<ML_DSA_PrivateKey>(rng, mode);
+   }
+#endif
+
 #if defined(BOTAN_HAS_HSS_LMS)
    if(alg_name == "HSS-LMS") {
       return std::make_unique<HSS_LMS_PrivateKey>(rng, params);
@@ -508,13 +582,21 @@ std::unique_ptr<Private_Key> create_private_key(std::string_view alg_name,
 #endif
 
 #if defined(BOTAN_HAS_SPHINCS_PLUS_WITH_SHA2) || defined(BOTAN_HAS_SPHINCS_PLUS_WITH_SHAKE)
-   if(alg_name == "SPHINCS+" || alg_name == "SphincsPlus-") {
+   if(alg_name == "SPHINCS+" || alg_name == "SphincsPlus") {
       auto sphincs_params = Sphincs_Parameters::create(params);
 
       return std::make_unique<SphincsPlus_PrivateKey>(rng, sphincs_params);
    }
 #endif
 
+#if defined(BOTAN_HAS_SLH_DSA_WITH_SHA2) || defined(BOTAN_HAS_SLH_DSA_WITH_SHAKE)
+   if(alg_name == "SLH-DSA") {
+      auto slh_dsa_params = SLH_DSA_Parameters::create(params);
+
+      return std::make_unique<SLH_DSA_PrivateKey>(rng, slh_dsa_params);
+   }
+#endif
+
 #if defined(BOTAN_HAS_XMSS_RFC8391)
    if(alg_name == "XMSS") {
       const auto xmss_oid = [&]() -> XMSS_Parameters::xmss_algorithm_t {
