@@ -14,12 +14,17 @@
 #include <botan/tls_algos.h>
 #include <botan/tls_ciphersuite.h>
 #include <botan/tls_exceptn.h>
+#include <botan/internal/os_utils.h>
 #include <botan/internal/stl_util.h>
 #include <optional>
 #include <sstream>
 
 namespace Botan::TLS {
 
+bool Policy::allow_ssl_key_log_file() const {
+   return false;
+}
+
 std::vector<Signature_Scheme> Policy::allowed_signature_schemes() const {
    std::vector<Signature_Scheme> schemes;
 
@@ -158,7 +163,7 @@ Group_Params Policy::default_dh_group() const {
 std::vector<Group_Params> Policy::key_exchange_groups() const {
    // Default list is ordered by performance
    return {
-#if defined(BOTAN_HAS_CURVE_25519)
+#if defined(BOTAN_HAS_X25519)
       Group_Params::X25519,
 #endif
 #if defined(BOTAN_HAS_X448)
@@ -226,7 +231,7 @@ void Policy::check_peer_key_acceptable(const Public_Key& public_key) const {
       expected_keylength = minimum_rsa_bits();
    } else if(algo_name == "DH") {
       expected_keylength = minimum_dh_group_size();
-   } else if(algo_name == "ECDH" || algo_name == "Curve25519" || algo_name == "X448") {
+   } else if(algo_name == "ECDH" || algo_name == "X25519" || algo_name == "X448") {
       expected_keylength = minimum_ecdh_group_size();
    } else if(algo_name == "ECDSA") {
       expected_keylength = minimum_ecdsa_group_size();
@@ -617,6 +622,7 @@ void Policy::print(std::ostream& o) const {
    print_bool(o, "allow_tls12", allow_tls12());
    print_bool(o, "allow_tls13", allow_tls13());
    print_bool(o, "allow_dtls12", allow_dtls12());
+   print_bool(o, "allow_ssl_key_log_file", allow_ssl_key_log_file());
    print_vec(o, "ciphers", allowed_ciphers());
    print_vec(o, "macs", allowed_macs());
    print_vec(o, "signature_hashes", allowed_signature_hashes());
