@@ -217,7 +217,8 @@ constexpr OutR concatenate(Rs&&... ranges)
 
          // fill the result buffer using a back-inserter
          return [&result](auto&& range) {
-            std::copy(std::ranges::begin(range), std::ranges::end(range), std::back_inserter(unpack(result)));
+            std::copy(
+               std::ranges::begin(range), std::ranges::end(range), std::back_inserter(unwrap_strong_type(result)));
          };
       } else {
          if constexpr((ranges::statically_spanable_range<Rs> && ... && true)) {
@@ -356,8 +357,16 @@ class scoped_cleanup {
 
       scoped_cleanup(const scoped_cleanup&) = delete;
       scoped_cleanup& operator=(const scoped_cleanup&) = delete;
-      scoped_cleanup(scoped_cleanup&&) = delete;
-      scoped_cleanup& operator=(scoped_cleanup&&) = delete;
+
+      scoped_cleanup(scoped_cleanup&& other) : m_cleanup(std::move(other.m_cleanup)) { other.disengage(); }
+
+      scoped_cleanup& operator=(scoped_cleanup&& other) {
+         if(this != &other) {
+            m_cleanup = std::move(other.m_cleanup);
+            other.disengage();
+         }
+         return *this;
+      }
 
       ~scoped_cleanup() {
          if(m_cleanup.has_value()) {
@@ -399,6 +408,64 @@ class StringLiteral {
       char value[N];
 };
 
+// TODO: C++23: replace with std::to_underlying
+template <typename T>
+   requires std::is_enum_v<T>
+auto to_underlying(T e) noexcept {
+   return static_cast<std::underlying_type_t<T>>(e);
+}
+
+// TODO: C++23 - use std::out_ptr
+template <typename T>
+[[nodiscard]] constexpr auto out_ptr(T& outptr) noexcept {
+   class out_ptr_t {
+      public:
+         constexpr ~out_ptr_t() noexcept {
+            m_ptr.reset(m_rawptr);
+            m_rawptr = nullptr;
+         }
+
+         constexpr out_ptr_t(T& outptr) noexcept : m_ptr(outptr), m_rawptr(nullptr) {}
+
+         out_ptr_t(const out_ptr_t&) = delete;
+         out_ptr_t(out_ptr_t&&) = delete;
+         out_ptr_t& operator=(const out_ptr_t&) = delete;
+         out_ptr_t& operator=(out_ptr_t&&) = delete;
+
+         [[nodiscard]] constexpr operator typename T::element_type **() && noexcept { return &m_rawptr; }
+
+      private:
+         T& m_ptr;
+         typename T::element_type* m_rawptr;
+   };
+
+   return out_ptr_t{outptr};
+}
+
+template <typename T>
+   requires std::is_default_constructible_v<T>
+[[nodiscard]] constexpr auto out_opt(std::optional<T>& outopt) noexcept {
+   class out_opt_t {
+      public:
+         constexpr ~out_opt_t() noexcept { m_opt = m_raw; }
+
+         constexpr out_opt_t(std::optional<T>& outopt) noexcept : m_opt(outopt) {}
+
+         out_opt_t(const out_opt_t&) = delete;
+         out_opt_t(out_opt_t&&) = delete;
+         out_opt_t& operator=(const out_opt_t&) = delete;
+         out_opt_t& operator=(out_opt_t&&) = delete;
+
+         [[nodiscard]] constexpr operator T*() && noexcept { return &m_raw; }
+
+      private:
+         std::optional<T>& m_opt;
+         T m_raw;
+   };
+
+   return out_opt_t{outopt};
+}
+
 }  // namespace Botan
 
 #endif
