@@ -33,6 +33,7 @@ def known_targets():
         'bsi',
         'codeql',
         'coverage',
+        'cross-alpha',
         'cross-android-arm32',
         'cross-android-arm64',
         'cross-android-arm64-amalgamation',
@@ -40,13 +41,18 @@ def known_targets():
         'cross-arm32-baremetal',
         'cross-arm64',
         'cross-arm64-amalgamation',
+        'cross-hppa64',
         'cross-i386',
         'cross-ios-arm64',
+        'cross-m68k',
+        'cross-mips',
         'cross-mips64',
         'cross-ppc32',
         'cross-ppc64',
-        'cross-s390x',
         'cross-riscv64',
+        'cross-s390x',
+        'cross-sh4',
+        'cross-sparc64',
         'cross-win64',
         'docs',
         'emscripten',
@@ -58,7 +64,9 @@ def known_targets():
         'limbo',
         'minimized',
         'nist',
+        'no_pcurves',
         'sanitizer',
+        'sde',
         'shared',
         'static',
         'valgrind',
@@ -91,9 +99,9 @@ class LoggingGroup:
             print("::endgroup::")
 
 def build_targets(target, target_os):
-    if target in ['shared', 'minimized', 'bsi', 'nist']:
+    if target in ['shared', 'minimized', 'bsi', 'nist', 'examples']:
         yield 'shared'
-    elif target in ['static', 'examples', 'fuzzers', 'cross-arm32-baremetal', 'emscripten']:
+    elif target in ['static', 'fuzzers', 'cross-arm32-baremetal', 'emscripten']:
         yield 'static'
     elif target_os in ['windows']:
         yield 'shared'
@@ -113,8 +121,10 @@ def build_targets(target, target_os):
         yield 'bogo_shim'
     if target in ['sanitizer'] and target_os not in ['windows']:
         yield 'bogo_shim'
-    if target in ['examples']:
+    if target in ['examples', 'amalgamation']:
         yield 'examples'
+    if target in ['valgrind', 'valgrind-full']:
+        yield 'ct_selftest'
 
 def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache,
                     root_dir, build_dir, test_results_dir, pkcs11_lib, use_gdb,
@@ -127,7 +137,7 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache,
 
     if target_os not in ['linux', 'osx', 'windows', 'freebsd']:
         print('Error unknown OS %s' % (target_os))
-        return (None, None, None)
+        return (None, None, None, None)
 
     if is_cross_target:
         if target_os == 'osx':
@@ -148,6 +158,7 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache,
 
     make_prefix = []
     test_prefix = []
+    pretest_cmd = []
     test_cmd = [os.path.join(build_dir, 'botan-test'),
                 '--data-dir=%s' % os.path.join(root_dir, 'src', 'tests', 'data'),
                 '--run-memory-intensive-tests']
@@ -197,12 +208,15 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache,
     if target in ['minimized']:
         flags += ['--minimized-build', '--enable-modules=system_rng,sha2_32,sha2_64,aes']
 
+    if target in ['no_pcurves']:
+        flags += ['--disable-modules=pcurves_impl']
+
     if target in ['amalgamation', 'cross-arm64-amalgamation', 'cross-android-arm64-amalgamation']:
         flags += ['--amalgamation']
 
     if target in ['bsi', 'nist']:
         # tls is optional for bsi/nist but add it so verify tests work with these minimized configs
-        flags += ['--module-policy=%s' % (target), '--enable-modules=tls12']
+        flags += ['--module-policy=%s' % (target), '--enable-modules=tls12', '--disable-deprecated-features']
 
     if target in ['docs']:
         flags += ['--with-doxygen', '--with-sphinx', '--with-rst2man']
@@ -230,6 +244,9 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache,
     if target in ['coverage', 'sanitizer', 'fuzzers']:
         flags += ['--unsafe-terminate-on-asserts']
 
+    if target in ['sde']:
+        test_prefix = ['sde', '-future', '--']
+
     if target in ['valgrind', 'valgrind-full']:
         flags += ['--with-valgrind']
 
@@ -240,6 +257,8 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache,
                        '--show-reachable=yes',
                        '--track-origins=yes']
 
+        pretest_cmd = ['python3', os.path.join(root_dir, 'src', 'ct_selftest', 'ct_selftest.py'), os.path.join(build_dir, 'botan_ct_selftest')]
+
         # valgrind is single threaded anyway
         test_cmd += ['--test-threads=1']
 
@@ -252,10 +271,11 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache,
                 'ed25519_sign', 'elgamal_decrypt', 'elgamal_encrypt', 'elgamal_keygen',
                 'ffi_dh', 'ffi_dsa', 'ffi_elgamal', 'frodo_kat_tests', 'hash_nist_mc',
                 'hss_lms_keygen', 'hss_lms_sign', 'mce_keygen', 'passhash9', 'pbkdf',
-                'pwdhash', 'rsa_encrypt', 'rsa_pss', 'rsa_pss_raw', 'scrypt',
-                'sphincsplus', 'sphincsplus_fors', 'sphincsplus_keygen', 'srp6_kat',
+                'pcurves_arith', 'pwdhash', 'rsa_encrypt', 'rsa_pss', 'rsa_pss_raw', 'scrypt',
+                'sphincsplus', 'sphincsplus_fors', 'slh_dsa_keygen', 'slh_dsa', 'srp6_kat',
                 'srp6_rt', 'unit_tls', 'x509_path_bsi', 'x509_path_rsa_pss',
-                'xmss_keygen', 'xmss_keygen_reference', 'xmss_sign', 'xmss_verify', 'xmss_verify_invalid'
+                'xmss_keygen', 'xmss_keygen_reference', 'xmss_sign', 'xmss_unit_tests',
+                'xmss_verify', 'xmss_verify_invalid',
             ]
 
             disabled_tests += slow_tests
@@ -343,10 +363,32 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache,
                 flags += ['--cpu=aarch64']
                 cc_bin = 'aarch64-linux-gnu-g++'
                 test_prefix = ['qemu-aarch64', '-L', '/usr/aarch64-linux-gnu/']
+            elif target == 'cross-alpha':
+                flags += ['--cpu=alpha']
+                cc_bin = 'alpha-linux-gnu-g++'
+                test_prefix = ['qemu-alpha', '-L', '/usr/alpha-linux-gnu/']
+                flags += ['--without-stack-protector'] # not supported
+            elif target == 'cross-sh4':
+                flags += ['--cpu=sh4']
+                cc_bin = 'sh4-linux-gnu-g++'
+                test_prefix = ['qemu-sh4', '-L', '/usr/sh4-linux-gnu/']
+            elif target == 'cross-m68k':
+                flags += ['--cpu=m68k']
+                cc_bin = 'm68k-linux-gnu-g++'
+                test_prefix = ['qemu-m68k', '-L', '/usr/m68k-linux-gnu/']
+            elif target == 'cross-hppa64':
+                flags += ['--cpu=hppa']
+                cc_bin = 'hppa-linux-gnu-g++'
+                test_prefix = ['qemu-hppa', '-L', '/usr/hppa-linux-gnu/']
+            elif target == 'cross-sparc64':
+                flags += ['--cpu=sparc64']
+                cc_bin = 'sparc64-linux-gnu-g++'
+                test_prefix = ['qemu-sparc64', '-L', '/usr/sparc64-linux-gnu/']
             elif target == 'cross-ppc32':
                 flags += ['--cpu=ppc32']
                 cc_bin = 'powerpc-linux-gnu-g++'
                 test_prefix = ['qemu-ppc', '-L', '/usr/powerpc-linux-gnu/']
+                test_cmd = None # qemu crashes ...
             elif target == 'cross-ppc64':
                 flags += ['--cpu=ppc64', '--with-endian=little']
                 cc_bin = 'powerpc64le-linux-gnu-g++'
@@ -359,6 +401,10 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache,
                 flags += ['--cpu=s390x']
                 cc_bin = 's390x-linux-gnu-g++'
                 test_prefix = ['qemu-s390x', '-L', '/usr/s390x-linux-gnu/']
+            elif target == 'cross-mips':
+                flags += ['--cpu=mips32', '--with-endian=big']
+                cc_bin = 'mips-linux-gnu-g++'
+                test_prefix = ['qemu-mips', '-L', '/usr/mips-linux-gnu/']
             elif target == 'cross-mips64':
                 flags += ['--cpu=mips64', '--with-endian=big']
                 cc_bin = 'mips64-linux-gnuabi64-g++'
@@ -379,7 +425,7 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache,
             flags += ['--with-commoncrypto']
 
         def add_boost_support(target, target_os):
-            if target in ['coverage', 'shared']:
+            if target in ['coverage', 'shared', 'amalgamation']:
                 return True
 
             if target == 'sanitizer' and target_os == 'linux':
@@ -406,6 +452,28 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache,
         if target_os == 'linux':
             flags += ['--with-lzma']
 
+        if is_running_in_github_actions() and 'BOTAN_TPM2_ENABLED' in os.environ:
+            flags += ['--with-tpm2']
+
+            if os.environ.get('BOTAN_TPM2_ENABLED') == 'test':
+                # run the TPM2 tests
+                test_cmd += ["--tpm2-tcti-name=%s" % os.getenv('BOTAN_TPM2_TCTI_NAME'),
+                             "--tpm2-tcti-conf=%s" % os.getenv('BOTAN_TPM2_TCTI_CONF'),
+                             "--tpm2-persistent-rsa-handle=%s" % os.getenv('BOTAN_TPM2_PERSISTENT_RSA_KEY_HANDLE'),
+                             "--tpm2-persistent-ecc-handle=%s" % os.getenv('BOTAN_TPM2_PERSISTENT_ECC_KEY_HANDLE'),
+                             "--tpm2-persistent-auth-value=%s" % os.getenv('BOTAN_TPM2_PERSISTENT_KEY_AUTH_VALUE')]
+            elif os.environ.get('BOTAN_TPM2_ENABLED') == 'build':
+                # build the TPM2 module but don't run the tests
+                # TCTI name 'disabled' is a special value that disables the TPM2 tests
+                #
+                # TODO: This is a hack. It would be great if `./botan-test --skip-tests=`
+                #       would also work for test categories like `tpm2`. Currently it
+                #       only works for individual test names.
+                test_cmd += ["--tpm2-tcti-name=disabled"]
+
+        if is_running_in_github_actions() and 'BOTAN_BUILD_WITH_JITTERENTROPY' in os.environ:
+            flags += ['--enable-modules=jitter_rng']
+
         if target in ['coverage']:
             flags += ['--with-tpm']
             test_cmd += ['--run-online-tests']
@@ -426,6 +494,9 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache,
 
     flags += ['--cc-bin=%s' % (cc_bin)]
 
+    if not pretest_cmd:
+        pretest_cmd = None
+
     if test_cmd is None:
         run_test_command = None
     else:
@@ -445,7 +516,7 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache,
         else:
             run_test_command = test_prefix + test_cmd
 
-    return flags, run_test_command, make_prefix
+    return flags, pretest_cmd, run_test_command, make_prefix
 
 def run_cmd(cmd, root_dir, build_dir):
     """
@@ -685,7 +756,8 @@ def main(args=None):
             'src/scripts/dev_tools/run_clang_tidy.py',
             'src/editors/vscode/scripts/bogo.py',
             'src/editors/vscode/scripts/common.py',
-            'src/editors/vscode/scripts/test.py']
+            'src/editors/vscode/scripts/test.py',
+            'src/ct_selftest/ct_selftest.py']
 
         # This has to run in the repository root to generate the correct
         # relative paths in the output. Otherwise GitHub Actions will not
@@ -702,7 +774,7 @@ def main(args=None):
         if options.test_results_dir:
             os.makedirs(options.test_results_dir)
 
-        config_flags, run_test_command, make_prefix = determine_flags(
+        config_flags, pretest_cmd, run_test_command, make_prefix = determine_flags(
             target, options.os, options.cpu, options.cc, options.cc_bin,
             options.compiler_cache, root_dir, build_dir, options.test_results_dir,
             options.pkcs11_lib, options.use_gdb, options.disable_werror,
@@ -731,9 +803,12 @@ def main(args=None):
             if target in ['coverage', 'fuzzers']:
                 make_targets += ['fuzzer_corpus_zip', 'fuzzers']
 
-            if target in ['examples']:
+            if target in ['examples', 'amalgamation']:
                 make_targets += ['examples']
 
+            if target in ['valgrind', 'valgrind-full']:
+                make_targets += ['ct_selftest']
+
             if target in ['coverage', 'sanitizer'] and options.os not in ['windows']:
                 make_targets += ['bogo_shim']
 
@@ -742,6 +817,9 @@ def main(args=None):
             if options.compiler_cache is not None:
                 cmds.append([options.compiler_cache, '--show-stats'])
 
+        if pretest_cmd is not None:
+            cmds.append(pretest_cmd)
+
         if run_test_command is not None:
             cmds.append(run_test_command)
 
@@ -788,6 +866,10 @@ def main(args=None):
         if root_dir != '.':
             python_tests.append('--test-data-dir=%s' % root_dir)
 
+        if is_running_in_github_actions() and os.environ.get('BOTAN_TPM2_ENABLED', 'no') == 'test':
+            python_tests.extend(["--tpm2-tcti-name=%s" % os.getenv('BOTAN_TPM2_TCTI_NAME'),
+                                 "--tpm2-tcti-conf=%s" % os.getenv('BOTAN_TPM2_TCTI_CONF')])
+
         if target in ['shared', 'coverage'] and not (options.os == 'windows' and options.cpu == 'x86'):
             cmds.append([py_interp, '-b'] + python_tests)
 
