@@ -510,11 +510,15 @@ Test::Result PK_Key_Agreement_Test::run_one_test(const std::string& header, cons
       try {
          kas = std::make_unique<Botan::PK_Key_Agreement>(*privkey, this->rng(), kdf, provider);
 
-         auto derived_key = kas->derive_key(key_len, pubkey).bits_of();
-         result.test_eq(provider, "agreement", derived_key, shared);
+         if(agreement_should_fail(header, vars)) {
+            result.test_throws("key agreement fails", [&] { kas->derive_key(key_len, pubkey); });
+         } else {
+            auto derived_key = kas->derive_key(key_len, pubkey).bits_of();
+            result.test_eq(provider, "agreement", derived_key, shared);
 
-         if(key_len == 0 && kdf == "Raw") {
-            result.test_eq("Expected size", derived_key.size(), kas->agreed_value_size());
+            if(key_len == 0 && kdf == "Raw") {
+               result.test_eq("Expected size", derived_key.size(), kas->agreed_value_size());
+            }
          }
       } catch(Botan::Lookup_Error&) {
          //result.test_note("Skipping key agreement with with " + provider);
@@ -576,19 +580,20 @@ std::vector<Test::Result> PK_Key_Generation_Test::run() {
    std::vector<Test::Result> results;
 
    for(const auto& param : keygen_params()) {
-      const std::string report_name = algo_name() + (param.empty() ? param : " " + param);
+      const auto algo = algo_name(param);
+      const std::string report_name = Botan::fmt("{}{}", algo, (param.empty() ? param : " " + param));
 
       Test::Result result(report_name + " keygen");
 
-      const std::vector<std::string> providers = possible_providers(algo_name());
+      const std::vector<std::string> providers = possible_providers(algo);
 
       if(providers.empty()) {
-         result.note_missing("provider key generation " + algo_name());
+         result.note_missing("provider key generation " + algo);
       }
 
       result.start_timer();
       for(auto&& prov : providers) {
-         auto key_p = Botan::create_private_key(algo_name(), this->rng(), param, prov);
+         auto key_p = Botan::create_private_key(algo, this->rng(), param, prov);
 
          if(key_p == nullptr) {
             result.test_failure("create_private_key returned null, should throw instead");
@@ -611,8 +616,8 @@ std::vector<Test::Result> PK_Key_Generation_Test::run() {
                            oid.value().to_string(),
                            key.object_identifier().to_string());
          } else {
-            const bool exception = name == "Kyber" || name == "FrodoKEM" || name == "SPHINCS+";
-
+            const bool exception = name == "Kyber" || name == "ML-KEM" || name == "ML-DSA" || name == "SLH-DSA" ||
+                                   name == "FrodoKEM" || name == "SPHINCS+";
             if(!exception) {
                result.test_failure("Keys name " + name + " does not map to an OID");
             }
@@ -889,6 +894,15 @@ class PK_API_Sign_Test : public Text_Based_Test {
 
          return result;
       }
+
+      bool skip_this_test([[maybe_unused]] const std::string& header, const VarMap&) override {
+   #if !defined(BOTAN_HAS_SLH_DSA_WITH_SHA2)
+         if(header == "SLH-DSA") {
+            return true;
+         }
+   #endif
+         return false;
+      }
 };
 
 BOTAN_REGISTER_TEST("pubkey", "pk_api_sign", PK_API_Sign_Test);
